@@ -69,7 +69,7 @@ class AsynConnect (asynchat.async_chat):
 		return new_asyncon
 	
 	def set_backend (self, backend_keep_alive = 10):
-		self.backend = True
+		#self.backend = True
 		self.keep_alive = backend_keep_alive
 		
 	def set_auth (self, auth):
@@ -275,7 +275,7 @@ class AsynConnect (asynchat.async_chat):
 		
 	def recv (self, buffer_size):
 		try:
-			data = self.socket.recv (buffer_size)			
+			data = self.socket.recv (buffer_size)
 			if not data:
 				self.handle_close (700, "Connection closed unexpectedly in recv")
 				return b''
@@ -291,7 +291,8 @@ class AsynConnect (asynchat.async_chat):
 	
 	def send (self, data):		
 		try:
-			numsent = self.socket.send (data)
+			numsent = self.socket.send (data)			
+			#print ('-------', numsent)
 			if numsent:
 				self.set_event_time ()
 			return numsent	
