@@ -11,15 +11,15 @@ import types
 from .pydns import Base, Type, Class, Lib, Opcode
 import random
 import threading
-from ...lib.athreads.socket_map import thread_safe_socket_map
+from ... import lifetime
 
 defaults = Base.defaults
 Base.DiscoverNameServers ()
 
-socket_map = thread_safe_socket_map ()
-
 class UDPClient (asynchat.async_chat):
 	protocol = "udp"
+	zombie_timeout = 1200
+	ac_in_buffer_size = 512
 	
 	def __init__ (self, addr, logger):
 		self.addr = addr
@@ -28,14 +28,17 @@ class UDPClient (asynchat.async_chat):
 		self.creation_time = time.time ()		
 		self.closed = False
 		self._timeouted = False
-		self.callbacks = {}
-		asynchat.async_chat.__init__ (self, None, socket_map)
+		self.requests = {}
+		self.callback = None
+		asynchat.async_chat.__init__ (self)
 							
 	def query (self, request, args, callback):	
+		if self.callback is None:
+			self.callback = callback
 		self.event_time = time.time ()		
 		self.header = None
 		
-		self.callbacks [request [:2]] = [callback, args, time.time ()]
+		self.requests [request [:2]] = [args, time.time ()]
 		qname = args ["name"].decode ("utf8")
 		args ['addr'] = self.addr
 		self.push (request)
@@ -74,22 +77,25 @@ class UDPClient (asynchat.async_chat):
 	def handle_expt (self):
 		self.handle_close ()
 		
-	def collect_incoming_data (self, data):		
+	def collect_incoming_data (self, data, id = None):
 		try:
-			callback, args, starttime = self.callbacks.pop (data [:2])				
+			args, starttime = self.requests.pop (id or data [:2])				
 		except KeyError:
+			# alerady timeouted
 			pass
-		else:
-			callback (args, data, not data)
+		else:	
+			self.callback (args, data, id is not None)
 			
 	def handle_close (self):	
-		for callback, args, starttime in self.callbacks.values ():
-			callback (args, b'', self._timeouted)				
+		for args, starttime in self.requests.values ():
+			self.callback (args, b'', True)
 		self.close ()
 
 		
 class TCPClient (UDPClient):
 	protocol = "tcp"
+	zombie_timeout = 10
+	ac_in_buffer_size = 65536
 	
 	def __init__ (self, addr, logger):		
 		UDPClient.__init__ (self, addr, logger)
@@ -119,7 +125,7 @@ class TCPClient (UDPClient):
 			
 	def found_terminator (self):	
 		if self.header:
-			self.callback (self.args, self.header + self.reply, self._timeouted)				
+			self.callback (self.args, self.header + self.reply, self._timeouted)
 			self.close ()
 			
 		else:
@@ -130,27 +136,38 @@ class TCPClient (UDPClient):
 	def handle_close (self):	
 		self.callback (self.args, b'', self._timeouted)				
 		self.close ()
+
 		
-		
-class Request:
-	id = 0
-	def __init__(self, name, **args):
-		self.req (name, **args)
+class RequestHandler:
+	def __init__(self, lock, logger):
+		self.lock = lock
+		self.logger = logger
+		self.id = 0
+	
+	def get_id (self):
+		with self.lock:
+			self.id += 1
+			if self.id == 32768:
+				self.id = 1	
+			return self.id
 		
 	def argparse (self, name, args):
 		args['name'] = name
+		if 'errors' not in args:
+			args ['errors'] = []
+			
 		for i in list(defaults.keys()):
 			if i not in args:
 				args[i]=defaults[i]
 		return args
 		
-	def req (self, name, **args):
+	def handle_request (self, name, **args):
 		global pool
 		
 		if isinstance (name, str):
 			name = name.encode ("utf8")
 		args = self.argparse (name, args)
-		
+
 		protocol = args ['protocol']
 		opcode = args ['opcode']
 		rd = args ['rd']
@@ -165,22 +182,18 @@ class Request:
 			qtype = args ['qtype']
 			
 		qname = args ['name']		
-		m = Lib.Mpacker()
-		Request.id += 1
-		if Request.id == 32768:
-			Request.id = 1
-		m.addHeader(Request.id,
+		m = Lib.Mpacker()		
+		m.addHeader(self.get_id (),
 			  0, opcode, 0, 0, rd, 0, 0, 0,
 			  1, 0, 0, 0)
 		
 		m.addQuestion (qname, qtype, Class.IN)
 		request = m.getbuf ()
 		
-		args ['retry'] += 1
-		conn = getattr (pool, args ['protocol']) ()
-		conn.query (request, args, self.processReply)
+		conn = getattr (pool, args ['protocol']) (args.get ('addr'))
+		conn.query (request, args, self.process_reply)
 		
-	def processReply (self, args, data, timeouted):		
+	def process_reply (self, args, data, timeouted):		
 		global pool
 		
 		err = None
@@ -188,27 +201,25 @@ class Request:
 		qname = args ['name'].decode ('utf8')
 		
 		if timeouted:
-			err = 'timeout'
-			
+			err = 'timeout'			
 		else:	
 			try:
 				if not data:
-					err = "no reply"
-					
+					err = "no-reply"					
 				else:	
 					if args ["protocol"] == "tcp":
 						header = data [:2]
 						if len (header) < 2:
-							err = '%s, EOF' % qname
+							err = 'EOF-invalid-header' % qname
 						count = Lib.unpack16bit(header)
 						reply = data [2: 2 + count]
 						if len (reply) != count:
-							err = "incomplete reply"					
+							err = "incomplete-reply"					
 					else:
 						reply = data
 					
 			except:
-				pool.logger.trace ()
+				self.logger.trace ()
 				err = 'exception'	
 			
 			if not err:
@@ -219,21 +230,25 @@ class Request:
 					if r.header ['tc']:
 						err = 'truncate'
 						args ['protocol'] = 'tcp'
-						pool.logger ('%s, trucated switch to TCP' % qname, 'warn')
+						self.logger ('%s, trucated switch to TCP' % qname, 'warn')
 						
 					else:
 						if r.header ['status'] != 'NOERROR':
-							pool.logger ('%s, status %s' % (qname, r.header ['status']), 'warn')
+							self.logger ('%s, status %s' % (qname, r.header ['status']), 'warn')
 						answers = r.answers
 						
 				except:
-					pool.logger.trace ()
+					self.logger.trace ()
 		
 		if err:
-			if args ['retry'] < 3:					
-				self.req (**args)					
+			if len (args ['errors']) < 2:
+				args ['errors'].append (err)
+				# switch protocol
+				args ['protocol'] = args ['protocol'] == "tcp" and "udp" or "tcp"
+				query (**args)
 				return
-			pool.logger ('%s, DNS %s error' % (qname, err), 'warn')
+				
+			self.logger ('%s, DNS %s errors' % (qname, args ['errors']), 'warn')
 		
 		callback = args.get ("callback", None)
 		if callback:
@@ -244,45 +259,106 @@ class Request:
 
 
 class Pool:
-	zombie_timeout = 2
-	def __init__ (self, servers, logger):
-		#self.servers = [(x, 53) for x in servers]
-		self.logger = logger
-		self.udps = [UDPClient ((x, 53), self.logger) for x in (servers * 2)]		
-		self.servers = [(x, 53) for x in servers]
+	query_timeout = 1
 	
-	def __len__ (self):	
-		for each in list (socket_map.values ()):
-			if isinstance (each, TCPClient):
-				return 1				
-			elif each.callbacks:
-				return 1
+	def __init__ (self, servers, logger):
+		self.logger = logger
+		self.lock = threading.Lock ()
+		self.handler = RequestHandler (self.lock, self.logger)
+		self.servers = [(x, 53) for x in servers]
+		self.udps = [UDPClient (x, self.logger) for x in self.servers]						
+		self.queue = []
+		
+	def add (self, item):
+		with self.lock:
+			self.queue.append (item)
+	
+	def has_job (self):
+		with self.lock:
+			for client in self.udps:
+				if len (client.requests):
+					return 1
 		return 0
 	
-	def maintern (self, now):
-		for each in list (socket_map.values ()):
-			if isinstance (each, TCPClient):
-				if now - each.event_time > self.zombie_timeout:
-					each.handle_timeout ()
-					
-			else:	
-				for id, (callback, args, starttime) in list (each.callbacks.items ()):
-					if now - starttime > self.zombie_timeout:						
-						# timeout
-						each.collect_incoming_data (b'')						
-				
-	def tcp (self):
-		addr = random.choice (self.servers)		
-		return TCPClient (addr, self.logger)
-		
-	def udp (self):
-		return random.choice (self.udps)
+	def jobs (self):
+		t = []		
+		with self.lock:
+			for client in self.udps:
+				t.extend (list (client.requests.keys ()))
+		return t	
 			
+	def pop_all (self):
+		# DNS query maybe not allowed delay between request and send
+		# maybe they just drop response packet for delaying
+		with self.lock:
+			queue, self.queue = self.queue [:], []
+				
+		count = len (queue)
+		while queue:
+			name, args = queue.pop (0)
+			self.handler.handle_request (name, **args)
+	
+		if (not count and not self.has_job ()):
+			return
+		
+		map = {}
+		with self.lock:
+			for client in self.udps:
+				map [client._fileno] = client
+		fds = list (map.keys ())
+		
+		# maybe 2 is enough
+		safeguard = count * 2
+		while self.has_job () and safeguard:
+			safeguard -= 1
+			asyncore.loop (0.1, map, count = 1)
+			if safeguard % 5 == 0:
+				self.maintern (time.time ())		
+		self.maintern (time.time ())
+							
+		for fd in fds:
+			if fd not in map:
+				# resync 
+				try: del asyncore.socket_map [fd]
+				except KeyError: pass	
+		
+	def maintern (self, now):
+		for client in self.udps:
+			for id, (args, starttime) in list (client.requests.items ()):
+				if now - starttime > self.query_timeout:
+					client.collect_incoming_data (b'', id)
+			
+	def tcp (self, exclude = None):
+		while 1:
+			addr = random.choice (self.servers)
+			if addr != exclude:
+				return TCPClient (addr, self.logger)
+		
+	def udp (self, exclude = None):
+		while 1:
+			conn = random.choice (self.udps)
+			if conn.addr != exclude:
+				return conn
+
+
+def query (name, **args):
+	global pool
+	
+	if not lifetime.EXHAUST_DNS:
+		return pool.handler.handle_request (name, **args)
+	pool.add ((name, args))
+
+	
+def pop_all ():	
+	global pool
+	pool.pop_all ()
+	
 		
 PUBLIC_DNS_SERVERS = [
 	'8.8.8.8', 
 	'8.8.4.4'
 ]
+
 pool = None			
 def create_pool (dns_servers, logger):
 	global pool, PUBLIC_DNS_SERVERS
@@ -307,26 +383,41 @@ def test_callback (ans):
 	pprint.pprint (ans)	
 	if testset:
 		item = testset.pop ()		
-		Request (item, protocol = "udp", callback = test_callback, qtype = "a")		
+		self.handler.handle_request (item, protocol = "udp", callback = test_callback, qtype = "a")
 
-	
+def _print (ans):
+	if ans:
+		print (ans[0]['name'], ans[-1]['data'])
+	else:
+		print ("FAILED")
+		
+		
 if __name__	== "__main__":
 	from aquests.lib import logger
 	import pprint
 	
 	create_pool (PUBLIC_DNS_SERVERS, logger.screen_logger ())
-	Request ("www.microsoft.com", protocol = "tcp", callback = print, qtype="a")
-	Request ("www.cnn.com", protocol = "tcp", callback = print, qtype="a")
-	Request ("www.gitlab.com", protocol = "tcp", callback = print, qtype="a")
-	Request ("www.alexa.com", protocol = "udp", callback = print, qtype="a")
-	Request ("www.yahoo.com", protocol = "udp", callback = print, qtype="a")
-	Request ("www.github.com", protocol = "udp", callback = print, qtype="a")
-	Request ("www.google.com", protocol = "udp", callback = print, qtype="a")
-	Request ("www.amazon.com", protocol = "udp", callback = print, qtype="a")
-	print (socket_map)
-	for i in range (10):
-		asyncore.loop (timeout = 0.1, map = socket_map, count = 1)
-		print (len (pool), socket_map)
+	for i in range (4):
+		#query ("www.microsoft.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.cnn.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.gitlab.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.alexa.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.yahoo.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.github.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.google.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.amazon.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.almec.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.alamobeauty.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.alphaworld.com", protocol = "udp", callback = _print, qtype="a")
+		#query ("www.allrightsales.com", protocol = "udp", callback = _print, qtype="a")
+		query ("www.glasteel.com", protocol = "udp", callback = _print, qtype="a")
+	
+	pop_all ()
+	print ('------------------------')	
+	while 1:
+		pop_all ()
+		asyncore.loop (timeout = 1, count = 1)
+		print ('UNFINISHED', pool.jobs ())
 		
 	
 	
\ No newline at end of file
