@@ -1,7 +1,7 @@
 from ...protocols.http import base_request_handler
 from h2.connection import H2Connection, GoAwayFrame
 from h2.exceptions import ProtocolError, NoSuchStreamError, StreamClosedError, FlowControlError
-from h2.events import DataReceived, ResponseReceived, StreamEnded, ConnectionTerminated, StreamReset, WindowUpdated, RemoteSettingsChanged
+from h2.events import TrailersReceived, DataReceived, ResponseReceived, StreamEnded, ConnectionTerminated, StreamReset, WindowUpdated, RemoteSettingsChanged
 from h2.errors import ErrorCodes
 from h2.settings import SettingCodes
 from ...lib import producers
@@ -16,6 +16,8 @@ try:
 except ImportError:
 	from io import BytesIO
 
+
+			
 class FakeAsynConnect:
 	collector = None
 	def __init__ (self, asyncon, logger):
@@ -59,8 +61,32 @@ class FakeAsynConnect:
 	def disconnect (self):
 		pass
 			
+
+class FlowControlWindow:
+	MIN_RFCW = 4096
+	MIN_IBFCW = 65535
+	INRCEMENT_DELTA = 1048576
 	
-class RequestHandler (base_request_handler.RequestHandler):
+	def reset_stream (self, stream_id, error_code = 3):
+		self.conn.reset_stream (stream_id, error_code = 3)				 
+		
+	def adjust_flow_control_window (self, stream_id):
+		if self.conn.inbound_flow_control_window < self.MIN_IBFCW:			
+			with self._clock:
+				self.conn.increment_flow_control_window (self.INRCEMENT_DELTA)
+				
+		rfcw = self.conn.remote_flow_control_window (stream_id)
+		if rfcw < self.MIN_RFCW:
+			try:
+				with self._clock:
+					self.conn.increment_flow_control_window (self.INRCEMENT_DELTA, stream_id)
+			except StreamClosedError:
+				pass
+			except:
+				self.reset_stream (stream_id, 3)
+	
+		
+class RequestHandler (base_request_handler.RequestHandler, FlowControlWindow):
 	# HTTP2 Adaptor Class	
 	# this class rely: asyncon - this class - http_reqyest_handler
 	def __init__ (self, handler):
@@ -84,7 +110,7 @@ class RequestHandler (base_request_handler.RequestHandler):
 			self.add_request (1, handler)
 			self._send_stream_id = 1 # nexet request stream_id will be 3
 		else:		    
-			self.conn.initiate_connection()
+			self.conn.initiate_connection()				
 			
 		self.send_data ()
 		self.asyncon.set_terminator (9)
@@ -99,11 +125,13 @@ class RequestHandler (base_request_handler.RequestHandler):
 		self.asyncon.use_sendlock ()
 		self._send_stream_id = -1
 		self.requests = {}
-		self.conn = H2Connection ()
+		self.conn = H2Connection ()		
+		self.conn.local_settings.initial_window_size = 65535
+		
 		self.buf = b""
-		self.rfile = BytesIO ()
-		self.frame_buf = self.conn.incoming_buffer
-		self.frame_buf.max_frame_size = self.conn.max_inbound_frame_size		
+		self.rfile = BytesIO ()		
+		self.frame_buf = self.conn.incoming_buffer		
+		self.frame_buf.max_frame_size = self.conn.max_inbound_frame_size
 		self.data_length = 0
 		self.current_frame = None
 	
@@ -136,8 +164,8 @@ class RequestHandler (base_request_handler.RequestHandler):
 	
 	def send_data (self):
 		with self._clock:
-			data_to_send = self.conn.data_to_send ()		
-		if data_to_send:
+			data_to_send = self.conn.data_to_send ()						
+		if data_to_send:			
 			self.asyncon.push (data_to_send)
 							
 	def handle_request (self, handler):
@@ -148,7 +176,7 @@ class RequestHandler (base_request_handler.RequestHandler):
 		
 		headers, content_encoded = handler.get_request_header ("2.0", False)
 		payload = handler.get_request_payload ()
-		producer = None		
+		producer = None
 		if payload:
 			if type (payload) is bytes:
 				producer = producers.globbing_producer (
@@ -160,22 +188,13 @@ class RequestHandler (base_request_handler.RequestHandler):
 		
 		header = h2header_producer (stream_id, headers, producer, self.conn, self._clock)		
 		self.asyncon.push (header)		
-		if producer:			
+		if producer:
 			payload = h2frame_producer (stream_id, 0, 1, producer, self.conn, self._clock)
+			# is it proper?
+			#payload = producers.ready_globbing_producer (payload)
 			self.asyncon.push (payload)
-	
-	def increment_flow_control_window (self, cl, stream_id = 0):
-		if not stream_id:
-			with self._clock:
-				self.conn.increment_flow_control_window (cl)
-				self.send_data ()				
-		else:	
-			with self._clock:
-				try: self.conn.increment_flow_control_window (cl, stream_id)
-				except StreamClosedError: pass
-				else: self.send_data ()	
 					
-	def collect_incoming_data (self, data):
+	def collect_incoming_data (self, data):		
 		if not data:  return
 		if self.data_length:
 			self.rfile.write (data)
@@ -251,16 +270,22 @@ class RequestHandler (base_request_handler.RequestHandler):
 		if h:
 			h.collect_incoming_data ("\r\n".join (jheaders).encode ("utf8"))
 			h.found_terminator ()
-		
 		# finally, make inactive on post, put request 
 		self.asyncon.set_active (False)
-		
+	
+	def handle_trailers (self, stream_id, headers):
+		respheader = self.get_handler (stream_id).response.header
+		for k, v in headers:			
+			respheader.append ("{}: {}".format (k.decode ("utf8"), v.decode ("utf8")))
+					
 	def handle_events (self, events):
-		for event in events:
-			#print (event)
+		for event in events:			
 			if isinstance(event, ResponseReceived):
 				self.handle_response (event.stream_id, event.headers)
-					
+			
+			elif isinstance(event, TrailersReceived):
+				self.handle_trailers (event.stream_id, event.headers)
+				
 			elif isinstance(event, StreamReset):
 				if event.remote_reset:				
 					h = self.get_handler (event.stream_id)
@@ -276,22 +301,13 @@ class RequestHandler (base_request_handler.RequestHandler):
 				h = self.get_handler (event.stream_id)
 				if h:
 					h.collect_incoming_data (event.data)
-					rfcw = self.conn.remote_flow_control_window (event.stream_id)
-					if rfcw <= 16384:
-						self.increment_flow_control_window (1048576, event.stream_id)
-			
-			elif isinstance (event, RemoteSettingsChanged):
-				try:
-					iws = event.changed_settings [SettingCodes.INITIAL_WINDOW_SIZE].new_value
-				except KeyError:
-					pass
-				else:		
-					self.increment_flow_control_window ((2 ** 31 - 1) - iws)
-				
+					self.adjust_flow_control_window (event.stream_id)
+													
 			elif isinstance(event, StreamEnded):
 				h = self.get_handler (event.stream_id)
 				if h:
 					self.remove_handler (event.stream_id)
-					h.found_terminator ()					
+					h.found_terminator ()
+					
 		self.send_data ()
 		
\ No newline at end of file
